from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.Archetypes.public import *
from Products.Classified.config import CLASS_TYPE
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

ClassifiedSchema = BaseSchema + Schema((
    StringField('id',
                required=1,
                mode="rw",
                accessor="getId",
                mutator="setId",
                default=None,
                widget=IdWidget(label = 'Unique Classified ID',
                                label_msgid="label_name",
                                description_msgid="help_name",
                                i18n_domain="plone"),
                ),

    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label = 'Title',
                                    label_msgid="label_title",
                                    description_msgid="help_title",
                                    i18n_domain="plone"),
                ),
    StringField('class_type',
                vocabulary=CLASS_TYPE,
                widget = SelectionWidget(label = 'Classified Type'),
               ),
        
    ObjectField('categories',
        required = 1,
        default = (),
        searchable = 1,
        index = 'KeywordIndex',
        vocabulary = '_voc_categories',
        enforceVocabulary = 1,
        multiValued = 1,
        widget = MultiSelectionWidget(label = 'Categories',
                                          description = 'Classified Categories.')),

    ImageField('image',
               default_output_type='image/jpeg',
               allowable_content_types=('image/*',),
               sizes = { 'small' : (160,120), 
                         'normal' : (400,300)
                       },
               widget=ImageWidget(label = 'Item Image - if available'),
               ),
    StringField('price',
                searchable=1,
                required=1,
                widget=StringWidget(label = 'Price'),
                ),
    StringField('quantity',
                searchable=0,
                default='1',
                widget=StringWidget(label = 'Quantity'),
                ),
    TextField('item_description',
              searchable=1,
              required=1,
              allowable_content_types=('text/plain',
                                       'text/structured',
                                       'text/html',),
              widget=TextAreaWidget(label = 'Description',
                                    description = 'Descipription of item(s) to sell or item(s) wanted'),
              ),
    StringField('contact_person',
                searchable=1,
                widget=StringWidget(label = 'Person to Contact'),
                ),
    StringField('contact_phone',
                widget=StringWidget(label = 'Contact Phone'),
                ),
    StringField('contact_email',
                searchable=1,
                validators=('isEmail'),
                widget=StringWidget(label = 'Contact Email Address'),
                ),
    ))


class Classified(BaseContent):
    """Classified Ad"""

    schema = ClassifiedSchema
    security = ClassSecurityInfo()

    # ugly fix
    expires = DateTime() + 1000
    effective = DateTime() - 1

    def _voc_categories(self):
        classtool = getToolByName(self, 'portal_classifieds')
        return DisplayList(classtool.getCategoriesVocabulary()).sortedByValue()

    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'classified_view',
        'permissions': (CMFCorePermissions.View,)
        },)

registerType(Classified)
