from Products.Archetypes.public import *
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.CMFCore.CMFCorePermissions import View
from Products.Classified.utils import isInsideOf, checkPublished
from Products.CMFCore.utils import getToolByName
from config import PROJECTNAME

ClassifiedCategorySchema = BaseFolderSchema

class ClassifiedCategory(BaseFolder):
    """Classified Category"""

    schema = ClassifiedCategorySchema
    security = ClassSecurityInfo()
    
    # ugly fix
    expires = DateTime() + 1000
    effective = DateTime() - 1
    
    factory_type_information = {'filter_content_types' : 1,
                                'allowed_content_types' : ('ClassifiedCategory',)}

    def _makeBreadcrumbs(self, classcatfolder):
        classcat = self
        if isInsideOf(classcat, classcatfolder):
            r = []
            while (classcat != classcatfolder):
                r.append(classcat.title_or_id())
                classcat = classcat.aq_parent
            r.reverse()
            return '/'.join(r)
        else:
            raise CategoryOutsideCategoryTree

    security.declareProtected(View, 'getCategoryBreadcrumbs')
    def getCategoryBreadcrumbs(self):
        """Return the Category Breadcrumbs from the Classified Category
           Folder root (informed on Classified Association Tool) until the
           location of this Category object"""
        pla = getToolByName(self, 'portal_classifieds')
        classcatfolder = pla.getClassifiedCategoryFolderObject()
        return self._makeBreadcrumbs(classcatfolder)

    # WARNING: Do not use searchCategoryClassifieds method on validCategory.
    # It will create a BIG performance overhead.

    security.declareProtected(View, 'validCategory')
    def validCategory(self, rolesSeeHiddenContent = ['Manager']):
        """Verify if a Category object is valid. To be valid, a
           Category must be published and have a published Classified object
           inside of it or one of its internal Category objects. It's
           a recursive method!"""
        pm = getToolByName(self, 'portal_membership')
        user = pm.getAuthenticatedMember()
        pc = getToolByName(self, 'portal_catalog')
        sr = pc.searchResults(portal_type = ['Classified'], getCategories = self.UID())
        for r in sr:
            obj = pc.getobject(r.data_record_id_)
            showHiddenFiles = user.has_role(rolesSeeHiddenContent, obj)
            if showHiddenFiles or checkPublished(self, obj):
                return 1
        # Recursive call to validate internal Categories. If you don't
        # have a valid Classified on this Category, but you have a valid
        # Classified into an internal Category, this Category must be valid.
        internal_categories = self.objectValues(['ClassifiedCategory'])
        for ic in internal_categories:
            if ic.validCategory(rolesSeeHiddenContent):
                return 1
        return None

    security.declareProtected(View, 'searchCategoryClassifieds')
    def searchCategoryClassifieds(self, rolesSeeHiddenContent = ['Manager']):
        """Search Classified objects associated to this Category object"""
        pm = getToolByName(self, 'portal_membership')
        user = pm.getAuthenticatedMember()
        pc = getToolByName(self, 'portal_catalog')
        sr = pc.searchResults(portal_type = ['Classified'], getCategories = self.UID())
        res = []
        for r in sr:
            obj = pc.getobject(r.data_record_id_)
            showHiddenFiles = user.has_role(rolesSeeHiddenContent, obj)
            if showHiddenFiles or checkPublished(self, obj):
                res.append(obj)
        return res
        
        
    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'browse_classifieds',
        'permissions': (CMFCorePermissions.View,)
        },)

registerType(ClassifiedCategory, PROJECTNAME)

class CategoryOutsideCategoryTree(Exception):
    pass