from zLOG import LOG, INFO
from Products.CMFCore.utils import UniqueObject
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import ManagePortal, View
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass

def log(summary = '', text = '', log_level = INFO):
    LOG('ClassifiedTool', log_level, summary, text)

class ClassifiedTool(UniqueObject, SimpleItem):
    """Classified Tool"""

    id = 'portal_classifieds'
    meta_type = 'Classified Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    manage_options = ((
        {'label' : 'Overview',
         'action' : 'manage_overview'},
        {'label' : 'Policies',
         'action' : 'manage_policies'},
        {'label' : 'Repository',
         'action' : 'manage_repository'}))

    security.declarePublic('manage_overview')
    manage_overview = PageTemplateFile('www/explainClassifiedTool',
        globals(), __name__ = 'manage_overview')


    security.declareProtected(ManagePortal, 'manage_repository')
    manage_repository = PageTemplateFile('www/manageRepository',
        globals(), __name__ = 'manage_repository')

    def __init__(self, batch_size = 30,
                 multiple_categories = 0, paginate_results = 0):
        self.classified_folder = None
        self.classified_category_folder = None
        self.batch_size = batch_size
        self.multiple_categories = multiple_categories
        self.paginate_results = paginate_results

    security.declareProtected(ManagePortal, 'setRepository')
    def setRepository(self, classified_folder, classified_category_folder,
                      REQUEST = None):
        """Change the Classified Folder path where Classified objects could be
           stored and shared between Members and the Classified Category
           Folder path where the Classified Categories are stored"""
        self.classified_folder = classified_folder
        self.classified_category_folder = classified_category_folder
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(REQUEST['URL1'] + '/manage_workspace')

    security.declareProtected(ManagePortal, 'setPolicies')
    def setPolicies(self, batch_size = 30,
                  multiple_categories = 0, paginate_results = 0,
                  REQUEST = None):
        """Change the Classified Policies"""
        self.batch_size = batch_size
        self.multiple_categories = multiple_categories
        self.paginate_results = paginate_results
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(REQUEST['URL1'] + '/manage_workspace')
            
    def _lookupObject(self, attr):
        o = getattr(self, attr)
        if o.startswith('/'):
            o = o[1:]
        return self.aq_parent.restrictedTraverse(o)

    security.declareProtected(View, 'getClassifiedFolderObject')
    def getClassifiedFolderObject(self):
        """Lookup Classified Folder"""
        return self._lookupObject('classified_folder')

    security.declareProtected(View, 'getClassifiedCategoryFolderObject')
    def getClassifiedCategoryFolderObject(self):
        """Lookup Classified Category Folder"""
        return self._lookupObject('classified_category_folder')

    security.declareProtected(View, 'getCategoriesVocabulary')
    def getCategoriesVocabulary(self):
        """Search Categories Vocabulary"""
        obj = self.getClassifiedCategoryFolderObject()
        return _searchCategories(obj)

    def _findEndSubTree(self, i, t):
        elem = None
        for n in range(i + 1, len(t)):
            if t[n]['level'] == t[i]['level']:
                elem = n
                break
        if elem is None:
            elem = len(t)
        return elem

    def _findInsertionPosition(self, l1, sl):
        pos = None
        for n in range(0, len(l1)):
            if l1[n]['level'] == sl[0]['level'] and \
               l1[n]['title'] > sl[0]['title']:
                pos = n
                break
        return pos

    def _mergeTrees(self, l1, l2):
        while l2:
            elem = self._findEndSubTree(0, l2)
            sl = l2[:elem]
            l2 = l2[elem:]
            pos = self._findInsertionPosition(l1, sl)
            if pos is None:
                l1.extend(sl)
            else:
                while sl:
                    l1.insert(pos, sl[0])
                    sl.pop(0)
                    pos += 1
        return l1

    security.declareProtected(View, 'MergeStatelessTrees')
    def MergeStatelessTrees(self, t1, t2):
        t1['list'] = self._mergeTrees(t1['list'], t2['list'])
        return t1

def _searchCategories(container):
    ret = []
    for obj in container.objectValues(['ClassifiedCategory']):
        ret.append((obj.UID(), obj.getCategoryBreadcrumbs()))
        ret.extend(_searchCategories(obj))
    return tuple(ret)

InitializeClass(ClassifiedTool)
